# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - EventAggregator Macro

    @copyright: 2008, 2009 by Paul Boddie <paul@boddie.org.uk>
    @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
                2005-2008 MoinMoin:ThomasWaldmann.
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin import wikiutil
import EventAggregatorSupport
import calendar

linkToPage = EventAggregatorSupport.linkToPage

try:
    set
except NameError:
    from sets import Set as set

Dependencies = ['pages']

# HTML-related functions.

def getColour(s):
    colour = [0, 0, 0]
    digit = 0
    for c in s:
        colour[digit] += ord(c)
        colour[digit] = colour[digit] % 256
        digit += 1
        digit = digit % 3
    return tuple(colour)

def getBlackOrWhite(colour):
    if sum(colour) / 3.0 > 127:
        return (0, 0, 0)
    else:
        return (255, 255, 255)

def getMonthActionQueryString(argname, month):
    if month is not None:
        return "%s=%04d-%02d" % ((argname,) + month)
    else:
        return ""

def getMonthQueryString(calendar_name, argname, month):
    if month is not None:
        return "%s-%s=%04d-%02d" % ((calendar_name, argname) + month)
    else:
        return ""

# Macro functions.

def execute(macro, args):

    """
    Execute the 'macro' with the given 'args': an optional list of selected
    category names (categories whose pages are to be shown), together with
    optional named arguments of the following forms:

      start=YYYY-MM     shows event details starting from the specified month
      start=current-N   shows event details relative to the current month
      end=YYYY-MM       shows event details ending at the specified month
      end=current+N     shows event details relative to the current month

      mode=calendar     shows a calendar view of events
      mode=list         shows a list of events by month
      mode=ics          provides iCalendar data for the events

      names=daily       shows the name of an event on every day of that event
      names=weekly      shows the name of an event once per week

      calendar=NAME     uses the given NAME to provide request parameters which
                        can be used to control the calendar view
    """

    request = macro.request
    fmt = macro.formatter
    page = fmt.page
    _ = request.getText

    # Interpret the arguments.

    try:
        parsed_args = args and wikiutil.parse_quoted_separated(args, name_value=False) or []
    except AttributeError:
        parsed_args = args.split(",")

    parsed_args = [arg for arg in parsed_args if arg]

    # Get special arguments.

    category_names = []
    calendar_start = None
    calendar_end = None
    mode = "calendar"
    name_usage = "daily"
    calendar_name = None

    for arg in parsed_args:
        if arg.startswith("start="):
            calendar_start = EventAggregatorSupport.getParameterMonth(arg[6:])

        elif arg.startswith("end="):
            calendar_end = EventAggregatorSupport.getParameterMonth(arg[4:])

        elif arg.startswith("mode="):
            mode = arg[5:]

        elif arg.startswith("names="):
            name_usage = arg[6:]

        elif arg.startswith("calendar="):
            calendar_name = arg[9:]

        else:
            category_names.append(arg)

    # Find request parameters to override settings.

    if calendar_name is not None:
        calendar_start = EventAggregatorSupport.getFormMonth(request, calendar_name, "start") or calendar_start
        calendar_end = EventAggregatorSupport.getFormMonth(request, calendar_name, "end") or calendar_end

    # Get the events.

    events, shown_events, all_shown_events, earliest, latest = \
        EventAggregatorSupport.getEvents(request, category_names, calendar_start, calendar_end)

    # Get a concrete period of time.

    first, last = EventAggregatorSupport.getConcretePeriod(calendar_start, calendar_end, earliest, latest)

    # Define some useful navigation months.

    if calendar_name is not None:
        span = EventAggregatorSupport.span(first, last)
        number_of_months = span[0] * 12 + span[1] + 1

        previous_month_start = EventAggregatorSupport.prevmonth(first)
        next_month_start = EventAggregatorSupport.nextmonth(first)
        previous_month_end = EventAggregatorSupport.prevmonth(last)
        next_month_end = EventAggregatorSupport.nextmonth(last)

        previous_set_start = EventAggregatorSupport.monthupdate(first, -number_of_months)
        next_set_start = EventAggregatorSupport.monthupdate(first, number_of_months)
        previous_set_end = EventAggregatorSupport.monthupdate(last, -number_of_months)
        next_set_end = EventAggregatorSupport.monthupdate(last, number_of_months)

    # Make a calendar.

    output = []

    # Output download controls.

    download_all_link = "action=EventAggregatorSummary&doit=1&%s" % (
        "&".join([("category=%s" % name) for name in category_names])
        )
    download_link = download_all_link + ("&%s&%s" % (
        getMonthActionQueryString("start", calendar_start),
        getMonthActionQueryString("end", calendar_end)
        ))
    subscribe_all_link = download_all_link + "&format=RSS"
    subscribe_link = download_link + "&format=RSS"

    output.append(fmt.div(on=1, css_class="event-controls"))
    output.append(fmt.span(on=1, css_class="event-download"))
    output.append(linkToPage(request, page, _("Download this view"), download_link))
    output.append(fmt.span(on=0))
    output.append(fmt.span(on=1, css_class="event-download"))
    output.append(linkToPage(request, page, _("Download this calendar"), download_all_link))
    output.append(fmt.span(on=0))
    output.append(fmt.span(on=1, css_class="event-download"))
    output.append(linkToPage(request, page, _("Subscribe to this view"), subscribe_link))
    output.append(fmt.span(on=0))
    output.append(fmt.span(on=1, css_class="event-download"))
    output.append(linkToPage(request, page, _("Subscribe to this calendar"), subscribe_all_link))
    output.append(fmt.span(on=0))
    output.append(fmt.div(on=0))

    # Output top-level information.

    if mode == "list":
        output.append(fmt.bullet_list(on=1, attr={"class" : "event-listings"}))

    # Visit all months in the requested range, or across known events.

    for year, month in EventAggregatorSupport.daterange(first, last):

        # Either output a calendar view...

        if mode == "calendar":

            # Output a month.

            output.append(fmt.table(on=1, attrs={"tableclass" : "event-month"}))

            output.append(fmt.table_row(on=1))
            output.append(fmt.table_cell(on=1, attrs={"class" : "event-month-heading", "colspan" : "7"}))

            # Either write a month heading or produce a link for navigable
            # calendars.

            month_label = _(EventAggregatorSupport.getMonthLabel(month))

            if calendar_name is not None:

                # Links to the previous set of months and to a calendar shifted
                # back one month.

                previous_set_link = "%s&%s" % (
                    getMonthQueryString(calendar_name, "start", previous_set_start),
                    getMonthQueryString(calendar_name, "end", previous_set_end)
                    )
                previous_month_link = "%s&%s" % (
                    getMonthQueryString(calendar_name, "start", previous_month_start),
                    getMonthQueryString(calendar_name, "end", previous_month_end)
                    )

                output.append(fmt.span(on=1, css_class="previous-month"))
                output.append(linkToPage(request, page, "<<", previous_set_link))
                output.append(fmt.text(" "))
                output.append(linkToPage(request, page, "<", previous_month_link))
                output.append(fmt.span(on=0))

                # Links to the next set of months and to a calendar shifted
                # forward one month.

                next_set_link = "%s&%s" % (
                    getMonthQueryString(calendar_name, "start", next_set_start),
                    getMonthQueryString(calendar_name, "end", next_set_end)
                    )
                next_month_link = "%s&%s" % (
                    getMonthQueryString(calendar_name, "start", next_month_start),
                    getMonthQueryString(calendar_name, "end", next_month_end)
                    )

                output.append(fmt.span(on=1, css_class="next-month"))
                output.append(linkToPage(request, page, ">", next_month_link))
                output.append(fmt.text(" "))
                output.append(linkToPage(request, page, ">>", next_set_link))
                output.append(fmt.span(on=0))

                # A link leading to this month being at the top of the calendar.

                full_month_label = "%s %s" % (month_label, year)
                end_month = EventAggregatorSupport.monthupdate((year, month), number_of_months - 1)

                month_link = "%s&%s" % (
                    getMonthQueryString(calendar_name, "start", (year, month)),
                    getMonthQueryString(calendar_name, "end", end_month)
                    )
                output.append(linkToPage(request, page, full_month_label, month_link))

            else:
                output.append(fmt.span(on=1))
                output.append(fmt.text(month_label))
                output.append(fmt.span(on=0))
                output.append(fmt.text(" "))
                output.append(fmt.span(on=1))
                output.append(fmt.text(unicode(year)))
                output.append(fmt.span(on=0))

            output.append(fmt.table_cell(on=0))
            output.append(fmt.table_row(on=0))

            # Weekday headings.

            output.append(fmt.table_row(on=1))

            for weekday in range(0, 7):
                output.append(fmt.table_cell(on=1, attrs={"class" : "event-weekday-heading"}))
                output.append(fmt.text(_(EventAggregatorSupport.getDayLabel(weekday))))
                output.append(fmt.table_cell(on=0))

            output.append(fmt.table_row(on=0))

            # Process the days of the month.

            start_weekday, number_of_days = calendar.monthrange(year, month)

            # The start weekday is the weekday of day number 1.
            # Find the first day of the week, counting from below zero, if
            # necessary, in order to land on the first day of the month as
            # day number 1.

            first_day = 1 - start_weekday

            while first_day <= number_of_days:

                # Find events in this week and determine how to mark them on the
                # calendar.

                week_start = (year, month, max(first_day, 1))
                week_end = (year, month, min(first_day + 6, number_of_days))

                week_coverage, week_events = EventAggregatorSupport.getCoverage(
                    week_start, week_end, shown_events.get((year, month), []))

                # Output a week, starting with the day numbers.

                output.append(fmt.table_row(on=1))

                for weekday in range(0, 7):
                    day = first_day + weekday
                    date = (year, month, day)

                    # Output out-of-month days.

                    if day < 1 or day > number_of_days:
                        output.append(fmt.table_cell(on=1, attrs={"class" : "event-day-heading event-day-excluded"}))
                        output.append(fmt.table_cell(on=0))

                    # Output normal days.

                    else:
                        if date in week_coverage:
                            output.append(fmt.table_cell(on=1, attrs={"class" : "event-day-heading event-day-busy"}))
                        else:
                            output.append(fmt.table_cell(on=1, attrs={"class" : "event-day-heading event-day-empty"}))

                        output.append(fmt.div(on=1))
                        output.append(fmt.span(on=1, css_class="event-day-number"))
                        output.append(fmt.text(unicode(day)))
                        output.append(fmt.span(on=0))
                        output.append(fmt.div(on=0))

                        # End of day.

                        output.append(fmt.table_cell(on=0))

                # End of day numbers.

                output.append(fmt.table_row(on=0))

                # Either generate empty days...

                if not week_events:
                    output.append(fmt.table_row(on=1))

                    for weekday in range(0, 7):
                        day = first_day + weekday
                        date = (year, month, day)

                        # Output out-of-month days.

                        if day < 1 or day > number_of_days:
                            output.append(fmt.table_cell(on=1, attrs={"class" : "event-day event-day-excluded"}))
                            output.append(fmt.table_cell(on=0))

                        # Output empty days.

                        else:
                            output.append(fmt.table_cell(on=1, attrs={"class" : "event-day event-day-empty"}))

                    output.append(fmt.table_row(on=0))

                # Or visit each set of scheduled events...

                else:
                    for coverage, events in week_events:

                        # Output each set.

                        output.append(fmt.table_row(on=1))

                        # Then, output day details.

                        for weekday in range(0, 7):
                            day = first_day + weekday
                            date = (year, month, day)

                            # Skip out-of-month days.

                            if day < 1 or day > number_of_days:
                                output.append(fmt.table_cell(on=1, attrs={"class" : "event-day event-day-excluded"}))
                                output.append(fmt.table_cell(on=0))
                                continue

                            # Output the day.

                            if date in coverage:
                                output.append(fmt.table_cell(on=1, attrs={"class" : "event-day event-day-busy"}))
                            else:
                                output.append(fmt.table_cell(on=1, attrs={"class" : "event-day event-day-empty"}))

                            # Get event details for the current day.

                            for event_page, event_details in events:
                                if not (event_details["start"] <= date <= event_details["end"]):
                                    continue

                                event_summary = EventAggregatorSupport.getEventSummary(event_page, event_details)

                                # Generate a colour for the event.

                                bg = getColour(event_page.page_name)
                                fg = getBlackOrWhite(bg)
                                style = ("background-color: rgb(%d, %d, %d); color: rgb(%d, %d, %d);" % (bg + fg))
                                hidden_style = ("background-color: rgb(%d, %d, %d); color: rgb(%d, %d, %d);" % (bg + bg))

                                css_classes = ["event-summary"]

                                if event_details["start"] == date:
                                    css_classes.append("event-starts")
                                    start_of_event = 1
                                else:
                                    start_of_event = 0

                                if event_details["end"] == date:
                                    css_classes.append("event-ends")

                                # Output the event.

                                if name_usage == "daily" or start_of_event or weekday == 0 or day == 1:
                                    hide_text = 0
                                else:
                                    hide_text = 1

                                output.append(fmt.div(on=1, css_class="event-summary-box"))

                                if not hide_text:
                                    output.append(fmt.div(on=1, css_class=(" ".join(css_classes)), style=style))
                                    output.append(linkToPage(request, event_page, event_summary))
                                    output.append(fmt.div(on=0))

                                    # Add a pop-up element for long summaries.

                                    output.append(fmt.div(on=1, css_class="event-summary-popup", style=style))
                                    output.append(linkToPage(request, event_page, event_summary))
                                    output.append(fmt.div(on=0))

                                else:
                                    output.append(fmt.div(on=1, css_class=(" ".join(css_classes)), style=hidden_style))
                                    output.append(fmt.text(event_summary))
                                    output.append(fmt.div(on=0))

                                output.append(fmt.div(on=0))

                            # End of day.

                            output.append(fmt.table_cell(on=0))

                        # End of set.

                        output.append(fmt.table_row(on=0))

                # Process the next week...

                first_day += 7

            # End of month.

            output.append(fmt.table(on=0))

        # Or output a summary view...

        elif mode == "list":

            month_label = _(EventAggregatorSupport.getMonthLabel(month))

            output.append(fmt.listitem(on=1, attr={"class" : "event-listings-month"}))
            output.append(fmt.div(on=1, attr={"class" : "event-listings-month-heading"}))
            output.append(fmt.span(on=1))
            output.append(fmt.text(month_label))
            output.append(fmt.span(on=0))
            output.append(fmt.text(" "))
            output.append(fmt.span(on=1))
            output.append(fmt.text(unicode(year)))
            output.append(fmt.span(on=0))
            output.append(fmt.div(on=0))

            output.append(fmt.bullet_list(on=1, attr={"class" : "event-month-listings"}))

            # Get the events in order.

            ordered_events = EventAggregatorSummary.getOrderedEvents(shown_events.get((year, month), []))

            # Show the events in order.

            for event_page, event_details in ordered_events:
                event_summary = EventAggregatorSupport.getEventSummary(event_page, event_details)

                output.append(fmt.listitem(on=1, attr={"class" : "event-listing"}))

                # Link to the page using the summary.

                output.append(fmt.paragraph(on=1))
                output.append(linkToPage(request, event_page, event_summary))
                output.append(fmt.paragraph(on=0))

                # Start and end dates.

                output.append(fmt.paragraph(on=1))
                output.append(fmt.span(on=1))
                output.append(fmt.text("%04d-%02d-%02d" % event_details["start"]))
                output.append(fmt.span(on=0))
                output.append(fmt.text(" - "))
                output.append(fmt.span(on=1))
                output.append(fmt.text("%04d-%02d-%02d" % event_details["end"]))
                output.append(fmt.span(on=0))
                output.append(fmt.paragraph(on=0))

                # Location.

                if event_details.has_key("location"):
                    output.append(fmt.paragraph(on=1))
                    output.append(fmt.text(event_details["location"]))
                    output.append(fmt.paragraph(on=1))

                # Topics.

                if event_details.has_key("topics") or event_details.has_key("categories"):
                    output.append(fmt.bullet_list(on=1, attr={"class" : "event-topics"}))

                    for topic in event_details.get("topics") or event_details.get("categories"):
                        output.append(fmt.listitem(on=1))
                        output.append(fmt.text(topic))
                        output.append(fmt.listitem(on=0))

                    output.append(fmt.bullet_list(on=0))

                output.append(fmt.listitem(on=0))

            output.append(fmt.bullet_list(on=0))

    # Output top-level information.

    # End of list view output.

    if mode == "list":
        output.append(fmt.bullet_list(on=0))

    return ''.join(output)

# vim: tabstop=4 expandtab shiftwidth=4
